@@ -782,7 +782,7 @@ static RSignBytes *r_sign_func_empty_mask(RAnal *a, RAnalFunction *fcn) {
 
 	// get size
 	RCore *core = a->coreb.core;
-	int maxsz = a->coreb.cfggeti (core, "zign.maxsz");
+	int maxsz = a->coreb.cfgGetI (core, "zign.maxsz");
 	r_list_sort (fcn->bbs, &bb_sort_by_addr);
 	ut64 ea = fcn->addr;
 	RAnalBlock *bb = (RAnalBlock *)fcn->bbs->tail->data;
@@ -925,7 +925,7 @@ static char *real_function_name(RAnal *a, RAnalFunction *fcn) {
 	ut64 addr = fcn->addr;
 	const char *name = fcn->name;
 	// resolve the manged name
-	char *res = a->coreb.cmdstrf (core, "is,vaddr/eq/0x%"PFMT64x",demangled/cols,a/head/1,:quiet", addr);
+	char *res = a->coreb.cmdStrF (core, "is,vaddr/eq/0x%"PFMT64x",demangled/cols,a/head/1,:quiet", addr);
 	if (res) {
 		r_str_trim (res);
 		if (*res) {
@@ -938,23 +938,20 @@ static char *real_function_name(RAnal *a, RAnalFunction *fcn) {
 
 R_API int r_sign_all_functions(RAnal *a, bool merge) {
 	R_RETURN_VAL_IF_FAIL (a, 0);
+	RCore *core = a->coreb.core;
+	RCons *cons = core->cons;
 	RAnalFunction *fcn = NULL;
 	RListIter *iter = NULL;
 	int count = 0;
 	r_list_sort (a->fcns, fcn_sort);
 	const RSpace *sp = r_spaces_current (&a->zign_spaces);
 	char *prev_name = NULL;
-	r_cons_break_push (NULL, NULL);
+	r_cons_break_push (cons, NULL, NULL);
 	RCoreBind cb = a->coreb;
-	RCore *core = cb.core;
-	bool do_mangled = cb.cfggeti (core, "zign.mangled");
-#if R2_USE_NEW_ABI
+	bool do_mangled = cb.cfgGetI (core, "zign.mangled");
 	bool zign_dups = a->opt.zigndups;
-#else
-	bool zign_dups = cb.cfggeti (core, "zign.dups");
-#endif
 	r_list_foreach_prev (a->fcns, iter, fcn) {
-		if (r_cons_is_breaked ()) {
+		if (r_cons_is_breaked (core->cons)) {
 			break;
 		}
 		char *realname = do_mangled? strdup (fcn->name): real_function_name (a, fcn);
@@ -982,7 +979,7 @@ R_API int r_sign_all_functions(RAnal *a, bool merge) {
 			prev_name = NULL;
 		}
 	}
-	r_cons_break_pop ();
+	r_cons_break_pop (cons);
 	free (prev_name);
 	return count;
 }
@@ -1588,7 +1585,7 @@ static void list_sanitise_warn(char *s, const char *name, const char *field) {
 		}
 		if (sanitized) {
 			R_LOG_INFO ("%s->%s needs to be sanitized", name, field);
-			r_warn_if_reached ();
+			R_WARN_IF_REACHED ();
 		}
 	}
 }
@@ -1910,8 +1907,7 @@ static bool listCB(RSignItem *it, void *user) {
 	if (it->collisions) {
 		list_sign_list (a, it->collisions, ctx->pj, ctx->format, R_SIGN_COLLISIONS, it->name);
 	} else if (ctx->format == 'j') {
-		pj_ka (ctx->pj, "collisions");
-		pj_end (ctx->pj);
+		list_sign_list (a, it->collisions, ctx->pj, ctx->format, R_SIGN_COLLISIONS, it->name);
 	}
 
 	// Vars
@@ -1948,7 +1944,7 @@ R_API void r_sign_list(RAnal *a, int format) {
 
 	{ // R2_600 - we need to pass addr as argument
 		RCore *core = a->coreb.core;
-		ut64 addr = core? core->offset: UT64_MAX;
+		ut64 addr = core? core->addr: UT64_MAX;
 		struct ctxListCB ctx = { a, 0, format, pj, addr};
 		r_sign_foreach (a, listCB, &ctx);
 	}
@@ -1992,7 +1988,7 @@ R_API const char *r_sign_type_to_name(int type) {
 	case R_SIGN_BBHASH:
 		return "bbhash";
 	default:
-		r_warn_if_reached ();
+		R_WARN_IF_REACHED ();
 		return "UnknownType";
 	}
 }
@@ -2848,11 +2844,13 @@ static inline bool suggest_check(RAnal *a, struct metric_ctx *ctx) {
 R_API int r_sign_metric_search(RAnal *a, RSignSearchMetrics *sm) {
 	R_RETURN_VAL_IF_FAIL (a && sm, -1);
 	RListIter *iter;
+	RCore *core = a->coreb.core;
+	RCons *cons = core->cons;
 	r_list_sort (a->fcns, fcn_sort);
-	r_cons_break_push (NULL, NULL);
+	r_cons_break_push (cons, NULL, NULL);
 	struct metric_ctx ctx = { 0, NULL, sm, NULL, NULL };
 	r_list_foreach (a->fcns, iter, ctx.fcn) {
-		if (r_cons_is_breaked ()) {
+		if (r_cons_is_breaked (cons)) {
 			break;
 		}
 		ctx.it = metric_build_item (sm, ctx.fcn);
@@ -2861,7 +2859,7 @@ R_API int r_sign_metric_search(RAnal *a, RSignSearchMetrics *sm) {
 		}
 		r_sign_item_free (ctx.it);
 	}
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 	free (ctx.suggest);
 	return ctx.matched;
 }
@@ -3052,11 +3050,15 @@ static int signdb_type(const char *file) {
 			t = SIGNDB_TYPE_KV;
 		}
 	}
+#if defined(__GNUC__)
 	// XXX looks like a false positive bug in gcc 9.4 (debian CI)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wfree-nonheap-object"
 	free (data);
 #pragma GCC diagnostic pop
+#else
+	free (data);
+#endif
 	return t;
 }
 
