@@ -1,10 +1,10 @@
-/* radare - LGPL - Copyright 2013-2024 - pancake */
+/* radare - LGPL - Copyright 2013-2025 - pancake */
 
 #include <r_cons.h>
 #include <r_util/r_assert.h>
+#include <math.h>
 
-#define useUtf8 (r_cons_singleton ()->use_utf8)
-#define useUtf8Curvy (r_cons_singleton ()->use_utf8_curvy)
+#define PI 3.14159265359
 
 #define W(y) r_cons_canvas_write (c, y)
 #define G(x, y) r_cons_canvas_gotoxy (c, x, y)
@@ -170,14 +170,16 @@ static bool attribute_delete_cb(void *user, const ut64 key, const void *value) {
 	return true;
 }
 
-R_API void r_cons_canvas_clear(RConsCanvas *c) {
+R_API void r_cons_canvas_clear(RConsCanvas *c, int flags) {
 	R_RETURN_IF_FAIL (c && c->b);
 	int y;
 	for (y = 0; y < c->h; y++) {
 		memset (c->b[y], '\n', c->bsize[y]);
 	}
-
 	ht_up_foreach (c->attrs, attribute_delete_cb, c->attrs);
+	if (flags != R_CONS_CANVAS_FLAG_DEFAULT) {
+		c->flags = flags;
+	}
 }
 
 R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y) {
@@ -219,16 +221,21 @@ R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y) {
 	return ret;
 }
 
-R_API RConsCanvas *r_cons_canvas_new(int w, int h) {
+R_API RConsCanvas *r_cons_canvas_new(RCons *cons, int w, int h, int flags) {
 	if (w < 1 || h < 1) {
 		return NULL;
 	}
 	RConsCanvas *c = R_NEW0 (RConsCanvas);
-	if (!c) {
-		return NULL;
-	}
+	c->cons = cons;
 	c->bgcolor = strdup (Color_RESET);
 	c->bsize = NULL;
+	if (flags == R_CONS_CANVAS_FLAG_DEFAULT) {
+		c->flags = 0;
+	} else if (flags == R_CONS_CANVAS_FLAG_INHERIT) {
+		c->flags = cons? r_cons_canvas_flags (cons): 0;
+	} else {
+		c->flags = flags;
+	}
 	c->blen = NULL;
 	int i = 0;
 	c->color = 0;
@@ -265,7 +272,7 @@ R_API RConsCanvas *r_cons_canvas_new(int w, int h) {
 		goto beach;
 	}
 	c->attr = Color_RESET;
-	r_cons_canvas_clear (c);
+	r_cons_canvas_clear (c, -1);
 	return c;
 beach:
 	r_str_constpool_fini (&c->constpool);
@@ -284,6 +291,7 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s) {
 	if (!c || !_s || !*_s || !R_BETWEEN (0, c->y, c->h - 1) || !R_BETWEEN (0, c->x, c->w - 1)) {
 		return;
 	}
+	RCons *cons = c->cons;
 	char *oos = strdup (_s);
 	char *os = r_str_ansi_resetbg (oos, c->bgcolor);
 	const char *s = os;
@@ -295,7 +303,7 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s) {
 
 	/* split the string into pieces of non-ANSI chars and print them normally,
 	** using the ANSI chars to set the attr of the canvas */
-	r_cons_break_push (NULL, NULL);
+	r_cons_break_push (cons, NULL, NULL);
 	do {
 		const char *s_part = set_attr (c, s);
 		ch = 0;
@@ -344,8 +352,8 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s) {
 			attr_x += utf8_len;
 		}
 		s += piece_len;
-	} while (*s && !r_cons_is_breaked ());
-	r_cons_break_pop ();
+	} while (*s && !r_cons_is_breaked (cons));
+	r_cons_break_pop (cons);
 	c->x = orig_x;
 	free (oos);
 	free (os);
@@ -427,7 +435,7 @@ R_API void r_cons_canvas_print_region(RConsCanvas *c) {
 	if (o) {
 		r_str_trim_tail (o);
 		if (*o) {
-			r_cons_print (o);
+			r_cons_print (c->cons, o);
 		}
 		free (o);
 	}
@@ -436,7 +444,7 @@ R_API void r_cons_canvas_print_region(RConsCanvas *c) {
 R_API void r_cons_canvas_print(RConsCanvas *c) {
 	char *o = r_cons_canvas_tostring (c);
 	if (o) {
-		r_cons_print (o);
+		r_cons_print (c->cons, o);
 		free (o);
 	}
 }
@@ -492,12 +500,10 @@ R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h) {
 	c->h = h;
 	c->x = 0;
 	c->y = 0;
-	r_cons_canvas_clear (c);
+	r_cons_canvas_clear (c, R_CONS_CANVAS_FLAG_DEFAULT);
 	return true;
 }
 
-#include <math.h>
-#define PI 3.1415
 R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, const char *color) {
 	if (color) {
 		c->attr = color;
@@ -525,14 +531,19 @@ R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, cons
 }
 
 R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color) {
-	const char *hline = useUtf8? RUNECODESTR_LINE_HORIZ : "-";
-	const char *vtmp = useUtf8? RUNECODESTR_LINE_VERT : "|";
+	// NOTE: As long as utf and curvy flags are tied to the canvas, we need to
+	// regenerate the canvas to get such changes now. before the kons refactoring
+	// this changed when cconfig settings were modified. not sure if its worth.
+	const bool useutf = c->flags & R_CONS_CANVAS_FLAG_UTF8;
+	const bool usecrv = c->flags & R_CONS_CANVAS_FLAG_CURVY;
+	const char *hline = useutf? RUNECODESTR_LINE_HORIZ : "-";
+	const char *vtmp = useutf? RUNECODESTR_LINE_VERT : "|";
 	RStrBuf *vline = r_strbuf_new (NULL);
 	r_strbuf_appendf (vline, Color_RESET"%s%s", color, vtmp);
-	const char *tl_corner = useUtf8 ? (useUtf8Curvy ? RUNECODESTR_CURVE_CORNER_TL : RUNECODESTR_CORNER_TL) : ".";
-	const char *tr_corner = useUtf8 ? (useUtf8Curvy ? RUNECODESTR_CURVE_CORNER_TR : RUNECODESTR_CORNER_TR) : ".";
-	const char *bl_corner = useUtf8 ? (useUtf8Curvy ? RUNECODESTR_CURVE_CORNER_BL : RUNECODESTR_CORNER_BL) : "`";
-	const char *br_corner = useUtf8 ? (useUtf8Curvy ? RUNECODESTR_CURVE_CORNER_BR : RUNECODESTR_CORNER_BR) : "'";
+	const char *tl_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_TL : RUNECODESTR_CORNER_TL) : ".";
+	const char *tr_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_TR : RUNECODESTR_CORNER_TR) : ".";
+	const char *bl_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_BL : RUNECODESTR_CORNER_BL) : "`";
+	const char *br_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_BR : RUNECODESTR_CORNER_BR) : "'";
 	int i, x_mod;
 	int roundcorners = 0;
 	char *row = NULL, *row_ptr;
@@ -640,3 +651,15 @@ R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCan
 		r_cons_canvas_line_diagonal (c, x, y, x2, y2, style);
 	}
 }
+
+R_API int r_cons_canvas_flags(RCons * R_NONNULL cons) {
+	R_RETURN_VAL_IF_FAIL (cons, 0);
+	int flags = 0;
+	if (cons->use_utf8) {
+		flags |= R_CONS_CANVAS_FLAG_UTF8;
+	}
+	if (cons->use_utf8_curvy) {
+		flags |= R_CONS_CANVAS_FLAG_CURVY;
+	}
+	return flags;
+}
