@@ -1,21 +1,18 @@
-/* radare2 - LGPL - Copyright 2017-2023 - condret, MaskRay */
+/* radare2 - LGPL - Copyright 2017-2024 - condret, MaskRay */
 
 #include <r_io.h>
-#include <stdlib.h>
-#include <r_util.h>
-#include <sdb/sdb.h>
 
 #define END_OF_MAP_IDS UT32_MAX
 R_IPI bool io_bank_has_map(RIO *io, const ut32 bankid, const ut32 mapid);
 
 static RIOMap *io_map_new(RIO* io, int fd, int perm, ut64 delta, ut64 addr, ut64 size) {
-	R_RETURN_VAL_IF_FAIL (io && io->maps, NULL);
+	R_RETURN_VAL_IF_FAIL (io, NULL);
 	const ut64 fd_size = r_io_fd_size (io, fd);
 	if ((!size) || (fd_size <= delta)) {
 		return NULL;
 	}
 	RIOMap* map = R_NEW0 (RIOMap);
-	if (!map || !r_id_storage_add (io->maps, map, &map->id)) {
+	if (!r_id_storage_add (&io->maps, map, &map->id)) {
 		free (map);
 		return NULL;
 	}
@@ -55,13 +52,13 @@ R_API bool r_io_map_remap(RIO *io, ut32 id, ut64 addr) {
 				r_io_bank_del_map (io, io->bank, newmap->id);
 			}
 			ut32 bankid;
-			r_id_storage_get_lowest (io->banks, &bankid);
+			r_id_storage_get_lowest (&io->banks, &bankid);
 			do {
 				if (bankid != io->bank && io_bank_has_map (io, bankid, id)) {
 					// TODO: use threads here
 					r_io_bank_map_add_top (io, bankid, newmap->id);
 				}
-			} while (r_id_storage_get_next (io->banks, &bankid));
+			} while (r_id_storage_get_next (&io->banks, &bankid));
 		} else {
 			// restore previous location and size if creation of newmap failed
 			r_io_map_set_begin (map, ofrom);
@@ -70,11 +67,11 @@ R_API bool r_io_map_remap(RIO *io, ut32 id, ut64 addr) {
 		}
 	}
 	ut32 bankid;
-	r_id_storage_get_lowest (io->banks, &bankid);
+	r_id_storage_get_lowest (&io->banks, &bankid);
 	do {
 		// TODO: use threads here
 		r_io_bank_update_map_boundaries (io, bankid, id, ofrom, oto);
-	} while (r_id_storage_get_next (io->banks, &bankid));
+	} while (r_id_storage_get_next (&io->banks, &bankid));
 	return true;
 }
 
@@ -104,11 +101,9 @@ static bool _map_free_cb(void *user, void *data, ut32 id) {
 
 R_API void r_io_map_init(RIO* io) {
 	R_RETURN_IF_FAIL (io);
-	if (io->maps) {
-		r_id_storage_foreach (io->maps, _map_free_cb, NULL);
-		r_id_storage_free (io->maps);
-	}
-	io->maps = r_id_storage_new (1, END_OF_MAP_IDS);
+	r_id_storage_foreach (&io->maps, _map_free_cb, NULL);
+	r_id_storage_fini (&io->maps);
+	r_id_storage_init (&io->maps, 1, END_OF_MAP_IDS);
 }
 
 // check if a map with exact the same properties exists
@@ -123,13 +118,13 @@ R_API bool r_io_map_exists(RIO *io, RIOMap *map) {
 
 // check if a map with specified id exists
 R_API bool r_io_map_exists_for_id(RIO *io, ut32 id) {
-	R_RETURN_VAL_IF_FAIL (io && io->maps, false);
+	R_RETURN_VAL_IF_FAIL (io, false);
 	return r_io_map_get (io, id);
 }
 
 R_API RIOMap* r_io_map_get(RIO *io, ut32 id) {
 	R_RETURN_VAL_IF_FAIL (io, false);
-	return r_id_storage_get (io->maps, id);
+	return r_id_storage_get (&io->maps, id);
 }
 
 R_API RIOMap *r_io_map_add(RIO *io, int fd, int perm, ut64 delta, ut64 addr, ut64 size) {
@@ -150,7 +145,7 @@ R_API RIOMap *r_io_map_add(RIO *io, int fd, int perm, ut64 delta, ut64 addr, ut6
 				return NULL;
 			}
 			if (!r_io_bank_map_add_top (io, io->bank, map[0]->id)) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 				return NULL;
 			}
@@ -159,7 +154,7 @@ R_API RIOMap *r_io_map_add(RIO *io, int fd, int perm, ut64 delta, ut64 addr, ut6
 		map[1] = io_map_new (io, fd, perm, delta, addr, size);
 		if (!map[1]) {
 			if (map[0]) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 			}
 			free (map[1]);
@@ -167,10 +162,10 @@ R_API RIOMap *r_io_map_add(RIO *io, int fd, int perm, ut64 delta, ut64 addr, ut6
 		}
 		if (!r_io_bank_map_add_top (io, io->bank, map[1]->id)) {
 			if (map[0]) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 			}
-			r_id_storage_delete (io->maps, map[1]->id);
+			r_id_storage_delete (&io->maps, map[1]->id);
 			free (map[1]);
 			return NULL;
 		}
@@ -197,7 +192,7 @@ R_API RIOMap *r_io_map_add_bottom(RIO *io, int fd, int perm, ut64 delta, ut64 ad
 				return NULL;
 			}
 			if (!r_io_bank_map_add_bottom (io, io->bank, map[0]->id)) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 				return NULL;
 			}
@@ -206,7 +201,7 @@ R_API RIOMap *r_io_map_add_bottom(RIO *io, int fd, int perm, ut64 delta, ut64 ad
 		map[1] = io_map_new (io, fd, perm, delta, addr, size);
 		if (!map[1]) {
 			if (map[0]) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 			}
 			free (map[1]);
@@ -214,10 +209,10 @@ R_API RIOMap *r_io_map_add_bottom(RIO *io, int fd, int perm, ut64 delta, ut64 ad
 		}
 		if (!r_io_bank_map_add_bottom (io, io->bank, map[1]->id)) {
 			if (map[0]) {
-				r_id_storage_delete (io->maps, map[0]->id);
+				r_id_storage_delete (&io->maps, map[0]->id);
 				free (map[0]);
 			}
-			r_id_storage_delete (io->maps, map[1]->id);
+			r_id_storage_delete (&io->maps, map[1]->id);
 			free (map[1]);
 			return NULL;
 		}
@@ -226,6 +221,7 @@ R_API RIOMap *r_io_map_add_bottom(RIO *io, int fd, int perm, ut64 delta, ut64 ad
 	return NULL;
 }
 
+// Confusing function name
 R_API RIOMap *r_io_map_get_paddr(RIO* io, ut64 paddr) {
 	R_RETURN_VAL_IF_FAIL (io, NULL);
 	RIOBank *bank = r_io_bank_get (io, io->bank);
@@ -259,29 +255,29 @@ R_API void r_io_map_reset(RIO* io) {
 }
 
 R_API void r_io_map_del(RIO *io, ut32 id) {
-	R_RETURN_IF_FAIL (io && io->maps);
-	RIOMap *map = (RIOMap *)r_id_storage_get (io->maps, id);
+	R_RETURN_IF_FAIL (io);
+	RIOMap *map = (RIOMap *)r_id_storage_get (&io->maps, id);
 	if (!map) {
 		return;
 	}
 	ut32 bankid = 0;
-	if (!r_id_storage_get_lowest (io->banks, &bankid)) {
+	if (!r_id_storage_get_lowest (&io->banks, &bankid)) {
 		R_LOG_ERROR ("Cannot get the lowest bankid");
 		return;
 	}
 	do {
 		// TODO: use threads for every bank, except the current bank (io->bank)
 		r_io_bank_del_map (io, bankid, id);
-	} while (r_id_storage_get_next (io->banks, &bankid));
-	r_id_storage_delete (io->maps, id);
+	} while (r_id_storage_get_next (&io->banks, &bankid));
+	r_id_storage_delete (&io->maps, id);
 	_map_free_cb (NULL, map, id);
 }
 
 //delete all maps with specified fd
 R_API bool r_io_map_del_for_fd(RIO* io, int fd) {
-	R_RETURN_VAL_IF_FAIL (io && io->maps, false);
+	R_RETURN_VAL_IF_FAIL (io, false);
 	ut32 map_id;
-	if (!r_id_storage_get_lowest (io->maps, &map_id)) {
+	if (!r_id_storage_get_lowest (&io->maps, &map_id)) {
 		return false;
 	}
 
@@ -289,7 +285,7 @@ R_API bool r_io_map_del_for_fd(RIO* io, int fd) {
 	bool cont;
 	do {
 		ut32 next = map_id;	// is this actually needed?
-		cont = r_id_storage_get_next (io->maps, &next);
+		cont = r_id_storage_get_next (&io->maps, &next);
 		RIOMap *map = r_io_map_get (io, map_id);
 		if (map->fd == fd) {
 			ret = true;
@@ -331,7 +327,7 @@ R_API bool r_io_map_priorize_for_fd(RIO *io, int fd) {
 R_API void r_io_map_cleanup(RIO* io) {
 	R_RETURN_IF_FAIL (io);
 	//remove all maps if no descs exist
-	if (!io->files) {
+	if (!io->files.data) {
 		r_io_map_fini (io);
 		r_io_map_init (io);
 		return;
@@ -346,14 +342,10 @@ static bool _clear_banks_cb(void *user, void *data, ut32 id) {
 
 R_API void r_io_map_fini(RIO* io) {
 	R_RETURN_IF_FAIL (io);
-	if (io->banks) {
-		r_id_storage_foreach (io->banks, _clear_banks_cb, NULL);
-	}
-	if (io->maps) {
-		r_id_storage_foreach (io->maps, _map_free_cb, NULL);
-		r_id_storage_free (io->maps);
-		io->maps = NULL;
-	}
+	r_id_storage_foreach (&io->banks, _clear_banks_cb, NULL);
+	r_id_storage_foreach (&io->maps, _map_free_cb, NULL);
+	r_id_storage_fini (&io->maps);
+	io->maps = (const RIDStorage){0};
 }
 
 R_API void r_io_map_set_name(RIOMap* map, const char* name) {
@@ -369,9 +361,6 @@ R_API void r_io_map_del_name(RIOMap* map) {
 
 R_API bool r_io_map_locate(RIO *io, ut64 *addr, const ut64 size, ut64 load_align) {
 	R_RETURN_VAL_IF_FAIL (io, false);
-	if (load_align == 0) {
-		load_align = 1;
-	}
 	return r_io_bank_locate (io, io->bank, addr, size, load_align);
 }
 
@@ -389,7 +378,7 @@ R_API RList* r_io_map_get_by_fd(RIO* io, int fd) {
 	RListIter *iter;
 	RIOMapRef *mapref;
 	r_list_foreach_prev (bank->maprefs, iter, mapref) {
-		RIOMap *map = (RIOMap *)r_id_storage_get (io->maps, mapref->id);
+		RIOMap *map = (RIOMap *)r_id_storage_get (&io->maps, mapref->id);
 		if (map->fd == fd) {
 			r_list_append (map_list, map);
 		}
@@ -417,14 +406,14 @@ R_IPI bool io_map_resize(RIO *io, ut32 id, ut64 newsize) {
 				r_io_bank_update_map_boundaries (io, io->bank, id, r_io_map_from (map), oto);
 			}
 			ut32 bankid;
-			r_id_storage_get_lowest (io->banks, &bankid);
+			r_id_storage_get_lowest (&io->banks, &bankid);
 			do {
 				if (bankid != io->bank && io_bank_has_map (io, bankid, id)) {
 					// TODO: use threads here
 					r_io_bank_update_map_boundaries (io, io->bank, id, r_io_map_from (map), oto);
 					r_io_bank_map_add_top (io, bankid, newmap->id);
 				}
-			} while (r_id_storage_get_next (io->banks, &bankid));
+			} while (r_id_storage_get_next (&io->banks, &bankid));
 		} else {
 			// restore previous size if creating newmap failed
 			r_io_map_set_size (map, osize);
@@ -434,13 +423,13 @@ R_IPI bool io_map_resize(RIO *io, ut32 id, ut64 newsize) {
 	}
 	r_io_map_set_size (map, newsize);
 	ut32 bankid;
-	r_id_storage_get_lowest (io->banks, &bankid);
+	r_id_storage_get_lowest (&io->banks, &bankid);
 	do {
 		if (io_bank_has_map (io, bankid, id)) {
 			// TODO: use threads here
 			r_io_bank_update_map_boundaries (io, bankid, id, r_io_map_from (map), oto);
 		}
-	} while (r_id_storage_get_next (io->banks, &bankid));
+	} while (r_id_storage_get_next (&io->banks, &bankid));
 	return true;
 }
 
@@ -563,9 +552,6 @@ R_API bool r_io_map_write_to_overlay(RIOMap *map, ut64 addr, const ut8 *buf, int
 	RRBNode *node = r_crbtree_find_node (map->overlay, &search_itv, _overlay_chunk_find, NULL);
 	if (!node) {
 		MapOverlayChunk *chunk = R_NEW0 (MapOverlayChunk);
-		if (!chunk) {
-			return false;
-		}
 		chunk->buf = R_NEWS (ut8, r_itv_size (search_itv));
 		chunk->itv = search_itv;
 		if (!chunk->buf || !r_crbtree_insert (map->overlay, chunk, _overlay_chunk_insert, NULL)) {
@@ -626,9 +612,6 @@ R_API bool r_io_map_write_to_overlay(RIOMap *map, ut64 addr, const ut8 *buf, int
 		}
 	}
 	chunk = R_NEW0 (MapOverlayChunk);
-	if (!chunk) {
-		return false;
-	}
 	chunk->buf = R_NEWS (ut8, r_itv_size (search_itv));
 	if (!chunk->buf) {
 		free (chunk);
@@ -747,3 +730,107 @@ R_API void r_io_map_drain_overlay(RIOMap *map) {
 	}
 	r_queue_free (q);
 }
+
+R_API void r_io_map_overlay_foreach(RIOMap *map, RIOMapOverlayForeach cb, void *user) {
+	R_RETURN_IF_FAIL (map && cb);
+	if (!map->overlay || !map->overlay->size) {
+		return;
+	}
+	RRBNode *node = r_crbtree_first_node (map->overlay);
+	if (!node) {
+		return;
+	}
+	do {
+		MapOverlayChunk *moc = node->data;
+		RInterval itv = {
+			moc->itv.addr + map->itv.addr,
+			moc->itv.size
+		};
+		cb (itv, moc->buf, user);
+	} while ((node = r_rbnode_next (node)), node);
+}
+
+static const char* metatypename[R_IO_MAP_META_TYPE_LAST] = {
+	[R_IO_MAP_META_TYPE_NONE] = "",
+	[R_IO_MAP_META_TYPE_HEAP] = "heap",
+	[R_IO_MAP_META_TYPE_STACK] = "stack",
+	[R_IO_MAP_META_TYPE_MMAP] = "mmap",
+	[R_IO_MAP_META_TYPE_MMIO] = "mmio",
+	[R_IO_MAP_META_TYPE_DMA] = "dma",
+	[R_IO_MAP_META_TYPE_JIT] = "jit",
+	[R_IO_MAP_META_TYPE_BSS] = "bss",
+	[R_IO_MAP_META_TYPE_SHARED] = "shared",
+	[R_IO_MAP_META_TYPE_KERNEL] = "kernel",
+	[R_IO_MAP_META_TYPE_GUARD] = "guard",
+	[R_IO_MAP_META_TYPE_NULL] = "null",
+	[R_IO_MAP_META_TYPE_GPU] = "gpu",
+	[R_IO_MAP_META_TYPE_TLS] = "tls",
+	[R_IO_MAP_META_TYPE_BUFFER] = "buffer",
+	[R_IO_MAP_META_TYPE_COW] = "cow",
+	[R_IO_MAP_META_TYPE_PAGETABLES] = "pagetables"
+};
+
+static const char *metaflagname[16] = {
+	"paged",
+	"private",
+	"persistent",
+	"aslr",
+	"swap",
+	"dep",
+	"enclave",
+	"compressed",
+	"encrypted",
+	"large",
+	0
+};
+
+R_API bool r_io_map_setattr_fromstring(RIOMap *map, const char *s) {
+	int i, maptype;
+	for (maptype = 0; maptype < R_IO_MAP_META_TYPE_LAST; maptype++) {
+		if (strstr (s, metatypename[maptype])) {
+			ut32 mapflag = 0;
+			for (i = 0; i < R_IO_MAP_META_FLAG_LAST; i++) {
+				if (strstr (s, metaflagname[i])) {
+					mapflag |= (1 << i);
+				}
+			}
+			return r_io_map_setattr (map, maptype, mapflag);
+		}
+	}
+	R_LOG_DEBUG ("invalid map type string");
+	return false;
+}
+
+R_API bool r_io_map_setattr(RIOMap *map, ut32 type, ut32 flags) {
+	if (type >= R_IO_MAP_META_TYPE_LAST) {
+		R_LOG_DEBUG ("invalid map type");
+		return false;
+	}
+	if (flags >= R_IO_MAP_META_FLAG_LAST) {
+		R_LOG_DEBUG ("invalid map flags");
+		return false;
+	}
+	map->meta = type | (flags << 16);
+	return true;
+}
+
+R_API char *r_io_map_getattr(RIOMap *map) {
+	ut32 maptype = map->meta & 0xffff;
+	ut32 mapflag = (map->meta > 16) & 0xffff;
+	if (maptype >= R_IO_MAP_META_TYPE_LAST) {
+		return false;
+	}
+	if (mapflag >= R_IO_MAP_META_FLAG_LAST) {
+		return false;
+	}
+	RStrBuf *sb = r_strbuf_new ("");
+	r_strbuf_append (sb, metatypename[maptype]);
+	int i = 0;
+	for (i = 0; i < 16; i++) {
+		if (mapflag & i) {
+			r_strbuf_append (sb, "+");
+			r_strbuf_append (sb, metaflagname[i]);
+		}
+	}
+	return r_strbuf_drain (sb);
+}
