@@ -1,75 +1,31 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake */
+/* radare - LGPL - Copyright 2009-2025 - pancake */
 
 #include <r_util.h>
 
 /* Validate if char is printable , why not use ISPRINTABLE() ?? */
 R_API bool r_name_validate_print(const char ch) {
 	// TODO: support utf8
-	if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || isdigit (ch)) {
+	if (isalpha (ch & 0xff) || isdigit (ch & 0xff)) {
 		return true;
 	}
-	switch (ch) {
-	case '(':
-	case ')':
-	case '[':
-	case ']':
-	case '<':
-	case '+':
-	case '-':
-	case '>':
-	case '$':
-	case '%':
-	case '@':
-	case ' ':
-	case '.':
-	case ',':
-	case ':':
-	case '_':
+	const char chars[] = "()[]<>+-$%@ .,:_";
+	if (strchr (chars, ch)) {
 		return true;
-	case '\b':
-	case '\t':
-	case '\n':
-	case '\r':
-		// must be replaced with ' ' and trimmed later
-		return false;
 	}
 	return false;
 }
 
 // used to determine if we want to replace those chars with '_' in r_name_filter()
 R_API bool r_name_validate_dash(const char ch) {
-	switch (ch) {
-	case ' ':
-	case '-':
-	case '_':
-	case '/':
-	case '\\':
-	case '(':
-	case ')':
-	case '[':
-	case ']':
-	case '<':
-	case '>':
-	case '!':
-	case '?':
-	case '$':
-	case ';':
-	case '%':
-	case '@':
-	case '`':
-	case ',':
-	case '"':
-		return true;
-	}
-	return false;
+	const char chars[] = " -_/\\()[]<>!?$;%@`,\"";
+	return strchr (chars, ch);
 }
 
 R_API bool r_name_validate_char(const char ch) {
-	if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || isdigit (ch)) {
+	if (isalpha (ch & 0xff) || isdigit (ch & 0xff)) {
 		return true;
 	}
 	return (ch == '.' || ch == ':' || ch == '_');
-	// return (ch == ';' || ch == '.' || ch == ':' || ch == '_');
 }
 
 R_API bool r_name_validate_first(const char ch) {
@@ -85,6 +41,7 @@ R_API bool r_name_validate_first(const char ch) {
 }
 
 R_API bool r_name_check(const char *s) {
+	R_RETURN_VAL_IF_FAIL (s, false);
 	if (!r_name_validate_first (*s)) {
 		return false;
 	}
@@ -97,21 +54,21 @@ R_API bool r_name_check(const char *s) {
 }
 
 static inline bool is_special_char(char n) {
-	return (n == 's' || n == 'b' || n == 'f' || n == 'n' || n == 'r' || n == 't' || n == 'v' || n == 'a');
+	const char chars[] = "sbfnrtva";
+	return strchr (chars, n);
 }
 
 R_API const char *r_name_filter_ro(const char *a) {
-	if (!a) {
-		return NULL;
-	}
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	while (*a++ == '_');
 	return a - 1;
 }
 
 // filter string for printing purposes
 R_API bool r_name_filter_print(char *s) {
+	R_RETURN_VAL_IF_FAIL (s, false);
 	char *es = s + strlen (s);
-	char *os = s;
+	bool valid = true;
 	while (*s && s < es) {
 		int us = r_utf8_size ((const ut8*)s);
 		if (us > 1) {
@@ -120,47 +77,32 @@ R_API bool r_name_filter_print(char *s) {
 		}
 		if (!r_name_validate_print (*s)) {
 			r_str_cpy (s, s + 1);
+			valid = false;
 		}
 		s++;
 	}
-	return os;
+	return valid;
 }
 
 R_API bool r_name_filter(char *s, int maxlen) {
+	R_RETURN_VAL_IF_FAIL (s, false);
 	if (R_STR_ISEMPTY (s)) {
 		return false;
 	}
-	// if maxlen == -1 : R_FLAG_NAME_SIZE
-	// maxlen is ignored, the function signature must change
-	// char *os = s;
 	size_t count = 0;
 	r_str_trim_head (s);
-	if (!r_name_validate_first (*s)) {
+	bool valid = r_name_validate_first (*s);
+	if (!valid) {
 		*s = '_';
 	}
-#if 0
-		while (*s) {
-			if (maxlen > 0 && count > maxlen) {
-				*s = 0;
-				break;
-			}
-			if (r_name_validate_char (*s)) {
-				break;
-			}
-			if (r_name_validate_dash (*s)) {
-				break;
-			}
-			r_str_cpy (s, s + 1);
-			count++;
-		}
-	}
-#endif
 	for (s++; *s; s++) {
 		if (maxlen > 0 && count > maxlen) {
+			valid = false;
 			*s = 0;
 			break;
 		}
 		if (*s == '\\') {
+			valid = false;
 			if (is_special_char (s[1])) {
 				*s = '_';
 			} else {
@@ -169,6 +111,7 @@ R_API bool r_name_filter(char *s, int maxlen) {
 			}
 		}
 		if (!r_name_validate_char (*s)) {
+			valid = false;
 			if (r_name_validate_dash (*s)) {
 				*s = '_';
 			} else {
@@ -178,10 +121,11 @@ R_API bool r_name_filter(char *s, int maxlen) {
 		}
 		count++;
 	}
-	return true;
+	return valid;
 }
 
 R_API char *r_name_filter_dup(const char *name) {
+	R_RETURN_VAL_IF_FAIL (name, NULL);
 	char *s = strdup (name);
 	r_name_filter (s, -1);
 	return s;
